/*
 * Copyright 2009-2010 Nanjing RedOrange ltd (http://www.red-orange.cn)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package redora.client.mvp;

import redora.client.util.Field;

import java.util.ArrayList;

/**
 * A View implementing FieldView allows to display {@link redora.client.util.Field}s on the panel.
 * The advantage is you can change the default generated list of fields by using
 * getFields() and moveField. For example:
 * <code>
 * MyView.getFields().add(MyObjectFields.extra);
 * MyView.getFields().set(new Field("Custom", "MyObject.custom", Field.Type.String), 1);
 * MyView.moveField(0, 3);
 * MyView.getFields().remove(2);
 * </code>
 *
 * @author Nanjing RedOrange (www.red-orange.cn)
 * @see redora.client.util.Field
 */
public interface FieldView {
    /** @return Current list of fields to be displayed. Use the standard list functions to change this list.*/
    ArrayList<Field> fields();
    /** Move a field item in the list. For example moveField(3, 0) moves the 4th item to the first position.*/
    void moveField(int from, int to);
}
